#
#  Copyright 1999, 2002-2005 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

proc perform_scan { record_list main } {

	set element [ $main.top_block.element_name get ]

	if { $element == "" } {
		set t "No element name has been specified."

		Mx::error_message "No Element Selected" $t "" ""

		return
	}

	set edge [ $main.top_block.edge_selection get ]

	if { $edge == "" } {
		set t \
	"An absorption edge has not been selected for element '$element'."

		Mx::error_message "No Edge Selected" $t "" ""

		return
	}

	set edge_energy [ $main.top_block.edge_energy get ]

	if { $edge_energy == "" } {
		set t \
	"The absorption edge energy has not yet been specified."

		Mx::error_message "No Edge Energy Selected" $t "" ""

		return
	}

	set type [ $main.middle_block.omenu get ]

	if { $type == "" } {
		set t "No scan type has been selected."

		Mx::error_message "No Scan Type Selected" $t "" ""

		return
	}

	set resolution [ $main.middle_block.ores get ]

	if { $resolution == "" } {
		set t "No scan resolution has been selected."

		Mx::error_message "No Scan Resolution Selected" $t "" ""

		return
	}

	set directory [ $main.bottom_block.directory get ]

	if { $directory == "" } {
		set t "No output directory has been selected."

		Mx::error_message "No Directory Selected" $t "" ""

		return
	}

	set type [ string tolower $type ]

	set resolution [ string tolower $resolution ]

	set upedge [ string toupper $edge ]

	set t "Starting scan '$type', resolution '$resolution' for $element "
	append t "$upedge edge at $edge_energy eV."

	log_puts $t

	if { $type == "absorption" } {

		perform_absorption_scan $record_list $element $edge \
					$edge_energy $resolution $directory

	} elseif { $type == "fluorescence" } {

		perform_fluorescence_scan $record_list $element $edge \
					$edge_energy $resolution $directory

	} else {
		set t "The scan type '$type' does not exist."

		Mx::error_message "Unrecognized Scan Type" $t "" ""

		return
	}
}

proc perform_absorption_scan { record_list element edge edge_energy \
					resolution directory } {

	set filename [ construct_scan_filename $directory $element $edge ]

	log_puts "Output file is '$filename'"

	# Compute the scan parameters.

	switch -exact -- $resolution {
		standard { set scan_name "mx_abs_standard" }
		short    { set scan_name "mx_abs_short" }
		fine     { set scan_name "mx_abs_fine" }
		coarse   { set scan_name "mx_abs_coarse" }
		default  { Mx::error_message "Unexpected Resolution" \
		"The scan resolution '$resolution' was not recognized." "" "" \
				; return }
	}
 
	if [ catch { $record_list get_record $scan_name } scan_record ] {
		set t "Scan record '$scan_name' does not exist.  "
		append t "IMCA GUI cannot perform the absorption scan "
		append t "without it."

		Mx::error_message "Scan Record Missing" $t "" ""

		return
	}

	$scan_record set_field "datafile_name" $filename

	perform_scan_main $record_list $edge_energy \
				$directory $scan_record $filename "absorption"
}

proc perform_fluorescence_scan { record_list element edge edge_energy \
					resolution directory } {

	set filename [ construct_scan_filename $directory $element $edge ]

	log_puts "Output file is '$filename'"

	# Compute the scan parameters.

	switch -exact -- $resolution {
		standard { set scan_name "mx_fl_standard" }
		short    { set scan_name "mx_fl_short" }
		fine     { set scan_name "mx_fl_fine" }
		coarse   { set scan_name "mx_fl_coarse" }
		default  { Mx::error_message "Unexpected Resolution" \
		"The scan resolution '$resolution' was not recognized." "" "" \
				; return }
	}
 
	if [ catch { $record_list get_record $scan_name } scan_record ] {
		set t "Scan record '$scan_name' does not exist.  "
		append t "IMCA GUI cannot perform the fluorescence scan "
		append t "without it."

		Mx::error_message "Scan Record Missing" $t "" ""

		return
	}

	$scan_record set_field "datafile_name" $filename

	perform_scan_main $record_list $edge_energy \
				$directory $scan_record $filename "fluorescence"
}

proc perform_scan_main { record_list edge_energy directory \
				scan_record filename scan_name } {

	puts "perform_scan_main invoked!"

	set main $IMCAGui::main_window

	#
	# Change the edge energy
	#

	set status \
	  [ catch { $record_list get_record "edge_energy" } edge_energy_record ]

	if { $status != 0 } {
		set t "Cannot find 'edge_energy' record."

		Mx::error_message "Error Setting 'edge_energy'" $t "" ""

		return
	}

	set status [ catch { $edge_energy_record send_variable $edge_energy } ]

	if { $status != 0 } {
		set t "Cannot set 'edge_energy' value."

		Mx::error_message "Error Setting 'edge_energy'" $t "" ""

		return
	}

	#
	# Change the monochromator enable flags.
	#

	save_and_set_enable_flags $record_list

	#
	# Disable feedback (on IMCA 17-BM).
	#

	if { $IMCAGui::beamline_name == "imca17bm" } {
		imca17bm_feedback_disable
	}

	#
	# Create a scan control window.
	#

	set window .scan_control

	frame $window
	pack $window -before .message_log

	label $window.scan_label -text "Current scan: "

	label $window.scan_name -text $scan_name

	pack $window.scan_label $window.scan_name -side left

	frame $window.sep -width 20
	pack $window.sep -side left -expand yes

	button $window.abort_scan -background red -activebackground red \
		-text "Abort Scan" -command [ itcl::code abort_scan_cmd ]

	pack $window.abort_scan -side right

	set sep .scan_control_sep

	frame $sep -height 2 -borderwidth 1 -relief sunken
	pack $sep -expand yes -fill x -pady 5 -side top -after $window

	update

	#
	# Run the scan
	#

	set scan_status [ catch { $scan_record perform_scan } ]

	#
	# Disable feedback (on IMCA 17-BM).
	#

	if { $IMCAGui::beamline_name == "imca17bm" } {
		imca17bm_feedback_enable
	}

#	if { $scan_status != 0 } {
#		set t \
#	"The scan '[ $scan_record cget -name ]' was abnormally terminated."
#
#		Mx::error_message "Scan Aborted" $t "" ""
#
#		return
#	}

	#
	# Restore the monochromator enable flags.
	#

	restore_enable_flags $record_list

	#
	# Update the Chooch datafile name
	#

	set chooch_filename_widget \
		$IMCAGui::experiment_window.bottom.input.chooch_filename

	$chooch_filename_widget clear

	$chooch_filename_widget insert end $filename

	#
	# Get rid of the scan control window.
	#

	update

	pack forget $window $sep
	destroy $window
	destroy $sep
}

proc abort_scan_cmd {} {
	log_puts "Scan abort requested."

	set IMCAGui::user_interrupt 1
}

proc construct_scan_filename { directory element edge } {

	set filename_stem "${element}_${edge}_scan"

	set pathstem [ file join $directory $filename_stem ]

	set pattern "${pathstem}.\[0-9\]\[0-9\]\[0-9\]"

	set glob_result [ glob -nocomplain -- $pattern ]

	set glob_result [ lsort $glob_result ]

	if { $glob_result == "" } {

		set filename "$pathstem.001"
	} else {
		set last_filename [ lindex $glob_result end ]

		set extension_raw [ file extension $last_filename ]

		if { [ scan $extension_raw {.%d} extension ] != 1 } {
			set t "Can't parse filename extension '$extension_raw'"

			Mx::error_message "Can't Parse Extension" $t "" ""

			return
		}

		if { $extension == "999" } {

			set new_extension "000"
		} else {

			set new_extension [ expr ( $extension + 1 ) ]

			set new_extension [ format {%03d} $new_extension ]
		}

		set filename "${pathstem}.${new_extension}"
	}

	return $filename
}

proc save_and_set_enable_flags { record_list } {

	if { $IMCAGui::beamline_name == "imca17bm" } {
		set focus_enabled_record \
			[ $record_list get_record "focus_enabled" ]

		set IMCAGui::focus_enabled \
			[ $focus_enabled_record receive_variable ]

		$focus_enabled_record send_variable 0

		set normal_enabled_record \
			[ $record_list get_record "normal_enabled" ]

		set IMCAGui::normal_enabled \
			[ $normal_enabled_record receive_variable ]

		$normal_enabled_record send_variable 0

		set parallel_enabled_record \
			[ $record_list get_record "parallel_enable" ]

		set IMCAGui::parallel_enabled \
			[ $parallel_enabled_record receive_variable ]

		$parallel_enabled_record send_variable 0

		set mbend_enabled_record \
			[ $record_list get_record "mbend_enabled" ]

		set IMCAGui::mbend_enabled \
			[ $mbend_enabled_record receive_variable ]

		$mbend_enabled_record send_variable 0

		set twist_enabled_record \
			[ $record_list get_record "twist_enabled" ]

		set IMCAGui::twist_enabled \
			[ $twist_enabled_record receive_variable ]

		$twist_enabled_record send_variable 0

		set omega_enabled_record \
			[ $record_list get_record "omega_enabled" ]

		set IMCAGui::omega_enabled \
			[ $omega_enabled_record receive_variable ]

		$omega_enabled_record send_variable 0
	}

	if { $IMCAGui::beamline_name == "ser22id" } {
		set id_ev_enabled_record \
			[ $record_list get_record "id_ev_enabled" ]

		set IMCAGui::id_ev_enabled \
			[ $id_ev_enabled_record receive_variable ]

		$id_ev_enabled_record send_variable 0

		set shutter_record \
			[ $record_list get_record "shutter" ]

		set IMCAGui::shutter_status \
			[ $shutter_record get_relay_status ]

		$shutter_record relay_command 0
	}

	return
}

proc restore_enable_flags { record_list } {

	if { $IMCAGui::beamline_name == "imca17bm" } {
		set focus_enabled_record \
			[ $record_list get_record "focus_enabled" ]

		$focus_enabled_record send_variable $IMCAGui::focus_enabled

		set normal_enabled_record \
			[ $record_list get_record "normal_enabled" ]

		$normal_enabled_record send_variable $IMCAGui::normal_enabled

		set parallel_enabled_record \
			[ $record_list get_record "parallel_enable" ]

		$parallel_enabled_record send_variable \
			$IMCAGui::parallel_enabled

		set mbend_enabled_record \
			[ $record_list get_record "mbend_enabled" ]

		$mbend_enabled_record send_variable \
			$IMCAGui::mbend_enabled

		set twist_enabled_record \
			[ $record_list get_record "twist_enabled" ]

		$twist_enabled_record send_variable \
			$IMCAGui::twist_enabled

		set omega_enabled_record \
			[ $record_list get_record "omega_enabled" ]

		$omega_enabled_record send_variable \
			$IMCAGui::omega_enabled
	}

	if { $IMCAGui::beamline_name == "ser22id" } {
		set id_ev_enabled_record \
			[ $record_list get_record "id_ev_enabled" ]

		$id_ev_enabled_record send_variable \
			$IMCAGui::id_ev_enabled

		set shutter_record \
			[ $record_list get_record "shutter" ]

		$shutter_record relay_command $IMCAGui::shutter_status
	}

	return
}

